# File: deletedbox.py
# Purpose: Deleted mail folder

# Almost identical to storebox. We could certainly remove this.

from gtk import *
import boxformats
import boxformats.configbox
from superbox import *
from ptk.big_edit_box import *
import ptk.misc_widgets
import pynei18n

class deletedbox(superbox):
	"""
	Deleted messages box.
	"""
	def get_flags(self):
		return BOX_DEL_MESSAGE

	def __init__(self, user, name, uid):
		"""
		Create new deleted mail folder.
		"""

		# Basic setup
		superbox.__init__(self, name, uid)
		self.expire_after = None
		self.export_format = (boxformats.BF_FLATFILEBOX, None)
		self.messages = []
		self.startup(user)

	def clean_4_save(self):
		"""
		Remove stuff we don't want saved to disk
		"""
		if self.__dict__.has_key("locked"):
			del self.locked
		if self.__dict__.has_key("changed"):
			del self.changed
		del self.io
		del self.messages

	def startup(self, user):
		"""
		Get an io module for loading/saving articles
		"""
		i = self.export_format[0]
		# this is a pretty piece of code :-)
		bf_class = boxformats.__dict__[boxformats.__all__[i]]. \
			   __dict__[boxformats.__all__[i]]
		self.io = bf_class(user, self)
		self.messages = self.io.get_contents()

	def get_menu(self):
		"""
		This box type specific options.
		"""
		return []

	def menu_chosen(self, _a, choice):
		"""
		Activate the chosen menu option.
		"""
		pass
		
	def getstats(self):
		"""
		Returns a tuple. (total msgs,).
		"""
		return (len(self.messages),)

	def setup(self, parent_user):
		"""
		Configure a deleted box.
		"""
		win = GtkWindow()
		win.set_title("Deleted Folder Setup")

		box = GtkVBox()
		win.add(box)
		box.show()

		notebook = GtkNotebook()
		notebook.set_tab_pos(POS_TOP)
		box.pack_start(notebook)
		notebook.show()

		# first page: misc settings
		settings_box1 = big_edit_box( self,
		      ( ("name", _("Name:"), VAR_TYPE_STRING, 0, 0),
			("expire_after", _("Expire after (days):"), VAR_TYPE_INTEGER, 0, None),
			("opts", _("Thread messages"), VAR_TYPE_PACKED_BIT_BOOLEAN, 0, OPT_THREAD) )
		)
		label = GtkLabel("Settings")
		notebook.append_page(settings_box1, label)
		settings_box1.show()

		# 2nd page: storage method
		label = GtkLabel(_("Mailbox Format"))
		formatbox = boxformats.configbox.configbox(self, parent_user)
		notebook.append_page(formatbox, label)
		formatbox.show()

		def save_changes(_button, self=self, parent_user=parent_user, win=win, \
				settings_box1 = settings_box1):
			# Extract info
			settings_box1.apply_changes()
			# update folder list
			self.changed = 1
			parent_user.update()

		def save_changes_close(_button, win=win, save_changes=save_changes):
			save_changes(_button)
			win.destroy()

		# Buttons at bottom
		lastbox = GtkHBox(spacing=5)
		lastbox.set_border_width(10)
		box.pack_start(lastbox, expand=FALSE)
		lastbox.show()

		button = ptk.misc_widgets.CancelButton()
		button.connect("clicked", win.destroy)
		lastbox.pack_end(button, expand=FALSE)
		button.show()

		button = ptk.misc_widgets.ApplyButton()
		button.connect("clicked", save_changes)
		lastbox.pack_end(button, expand=FALSE)
		button.show()

		button = ptk.misc_widgets.OkButton()
		button.connect("clicked", save_changes_close)
		lastbox.pack_end(button, expand=FALSE)
		button.show()

		win.show()

