#
# Incoming mail filters.
#

CONDITION_EQUAL =	0
CONDITION_NOTEQUAL =	1
CONDITION_CONTAINS =	2
CONDITION_LARGERTHAN =	3
CONDITION_REGEX_MATCH = 4
CONDITION_MATCH_ALL =	5

ACTION_MOVE = 0
ACTION_LEAVE = 1
ACTION_DELETE = 2
ACTION_FORWARD = 3
ACTION_LEAVE_COLLECT = 4

f_headers = [ "to", "from", "subject", "x-from_", "envelope-to" ]

f_conditions = [ "equals", "not equals", "contains", "size > (Kb)", "regex match", "match all" ]

f_actions = [ "Move to folder:",
	    "Leave on server",
            "Delete from server",
            "Forward to address:",
	    "Collect & leave on server"]

import time
import re
import string
from gtk import *
import ptk.folder_tree
from pyneheaders import *
import pynemsg
import boxtypes
import pynei18n
from boxtypes import superbox

# a filter is a tuple like so:
# ( string name, string header to match, integer condition,
#   string value to match, integer action, string action argument )
# eg: ( "Ignore annoying guy", "from", CONDITION_EQUAL, "twat@aol.com",
#       ACTION_MOVE, <Deleted folder UID as string> )
# will move mail from 'twat@aol.com' to the deleted folder

def filter_collect(parent_user, _mailbox, connection, index):
	"""
	Collect message 'index' from `connection`, applying
	filters.
	"""
	# return message
	r = ""
	# download headers only ATM
	msg = pynemsg.pynemsg()
	
	head = _mailbox.remote_get_header(connection, index)
	if head == None:
		return ""
	msg.body = head + "\n\n"
	# have we appended the body yet?
	gotbody = 0
	# Parse headers
	msg.parseheaders(parent_user, headers_only=1)
	msg.date_received = time.gmtime(time.time())
	# Does it match any filters (multiple matches possible)
	nomatches = 1
	for filtr in _mailbox.filters:
		name, head, cond, carg, actn, aarg = filtr
		match = 0
		# this condition tests not filthy headers...
		if cond != CONDITION_LARGERTHAN:
			if not msg.headers.has_key(head):
				continue
			head = msg.headers[head]

		if cond == CONDITION_LARGERTHAN:
			# get body and check size
			if isinstance(_mailbox, boxtypes.nntpbox.newsgroup):
				# don't bother if we are only collecting headers
				if (_mailbox.opts & superbox.OPT_HEADERS_ONLY):
					continue
				if gotbody == 0:
					body = _mailbox.remote_get_body(connection, index)
					msg.opts = msg.opts & (~MSG_NO_BODY)
					msg.body = msg.body + body
					gotbody = 1
				## including headers XXX
				if len(msg.body) > (1024*int(carg)):
					match = 1
			elif isinstance(_mailbox, boxtypes.mailbox.mailbox):
				# get message size from our {id: size} message  dict
				size = _mailbox.pending_msgs[index]
				if int(size) > (1024*int(carg)):
					match = 1
		elif cond == CONDITION_EQUAL:
			if head == carg:
				match = 1
		elif cond == CONDITION_NOTEQUAL:
			if head != carg:
				match = 1
		elif cond == CONDITION_CONTAINS:
			if string.find(head, carg) != -1:
				match = 1
		elif cond == CONDITION_REGEX_MATCH:
			if re.match(carg, head) != None:
				match = 1
		elif cond == CONDITION_MATCH_ALL:
			match = 1

		if match == 1:
			r = "\""+name+"\" "+_("filter match.")
			nomatches = 0

		if match == 1:
			# It matched a filter. do action
			if actn == ACTION_MOVE:
				# get body
				if (gotbody == 0) and not (_mailbox.opts & superbox.OPT_HEADERS_ONLY):
					body = _mailbox.remote_get_body(connection, index)
					msg.body = msg.body + body
					msg.opts = msg.opts & (~MSG_NO_BODY)
					gotbody = 1
				# get folder to move to, from UID
				# String or integer UID. great implementation, man :-{
				try:
					uid = int(aarg)
				except ValueError:
					uid = aarg
				folder = parent_user.get_folder_by_uid(uid)
				if folder == None:
					print "FILTER: Error. move_to folder not found."
					print "FILTER: "+str(filtr)
		
					_mailbox.io.save_article(msg)
					_mailbox.messages.append(msg.headers["message-id"])
					_mailbox.remote_delete(connection, index)
				else:
					folder.io.save_article(msg)

					# Add message to mailbox
					folder.messages.append(msg.headers["message-id"])

					# Delete from server
					_mailbox.remote_delete(connection, index)

					# Mark folder for update
					folder.changed = 1
				# some obsoleted comment clinging to existence
				return r

			elif actn == ACTION_LEAVE:
				# leave on server. don't store
				return r
			elif actn == ACTION_DELETE:
				# Delete from server. don't store
				_mailbox.remote_delete(connection, index)
				return r
			elif actn == ACTION_FORWARD:
				# get body
				if (gotbody == 0) and not (_mailbox.opts & superbox.OPT_HEADERS_ONLY):
					body = _mailbox.remote_get_body(connection, index)
					msg.body = msg.body + body
					msg.opts = msg.opts & (~MSG_NO_BODY)
					gotbody = 1
				msg.parseheaders(parent_user)
				outbox = parent_user.get_folder_by_uid("outbox")
				outbox._reply(parent_user, _mailbox, _mailbox, msg, \
				    REPLY_FORWARD, send_to=aarg, dont_open_editbox=1)
				# delete from server
				_mailbox.remote_delete(connection, index)
				return r
			elif actn == ACTION_LEAVE_COLLECT:
				# collect but leave on server
				if (gotbody == 0) and not (_mailbox.opts & superbox.OPT_HEADERS_ONLY):
					body = _mailbox.remote_get_body(connection, index)
					msg.body = msg.body + body
					msg.opts = msg.opts & (~MSG_NO_BODY)
					gotbody = 1
				_mailbox.io.save_article(msg)

				# Add message to mailbox
				_mailbox.messages.append(msg.headers["message-id"])
				return r
	if nomatches == 1:
		# default action. store to mailbox and trash
		# from server
		if (gotbody == 0) and not (_mailbox.opts & superbox.OPT_HEADERS_ONLY):
			body = _mailbox.remote_get_body(connection, index)
			msg.body = msg.body + body
			msg.opts = msg.opts & (~MSG_NO_BODY)
			gotbody = 1
		_mailbox.io.save_article(msg)

		# Add message to mailbox
		_mailbox.messages.append(msg.headers["message-id"])

		# Delete from server
		_mailbox.remote_delete(connection, index)
		return r

class filter_editbox:
	"""
	A GtkBox full of shit for editing a mailbox.filters
	"""

	def sel_folder(self, w, event, tree):
		folder = self.parent_user.get_folder_by_uid(tree.selected[0])
		if folder != None:
			self.aarg_in.set_text(folder.name)
			self.f_aarg = folder.uid
		self.update_state()

	def ch_condition(self, item):
		self.f_cond = int(item.get_data("0"))
	
	def ch_header(self, item):
		#self.header_in.set_text(item.get_data("0"))
		self.f_head = item.get_data("0")
		self.update_state()

	def update_state(self):
		# f_actn has changed. update widgets.
		# also f_head
		if self.f_head == len(f_headers):
			self.header_in.show()
		else:
			self.header_in.hide()
		
		if self.f_actn == ACTION_MOVE:
			self.folder_list.show()
			self.aarg_in.show()
			self.aarg_in.set_sensitive(FALSE)
			# No folder selected to move to. BAD!!
			folder = self.user.get_folder_by_uid(self.f_aarg)
			if folder == None:
				self.aarg_in.set_text("")
				self.apply_button.set_sensitive(FALSE)
				self.ok_button.set_sensitive(FALSE)
			else:
				self.aarg_in.set_text(folder.name)
				self.apply_button.set_sensitive(TRUE)
				self.ok_button.set_sensitive(TRUE)
		elif self.f_actn == ACTION_FORWARD:
			self.folder_list.hide()
			self.aarg_in.set_text(self.f_aarg)
			self.aarg_in.show()
			self.aarg_in.set_sensitive(TRUE)
		elif self.f_actn == ACTION_LEAVE or self.f_actn == ACTION_DELETE \
				or self.f_actn == ACTION_LEAVE_COLLECT:
			self.folder_list.hide()
			self.aarg_in.hide()

	def ch_action(self, item):
		self.f_actn = int(item.get_data("0"))
		self.update_state()

	def __init__(self, user, filter_list_box, index):
		import copy
		self.user = user
		
		self.filter = copy.deepcopy(filter_list_box.filters[index])
		self.filter_index = index
		self.parent_user = filter_list_box.parent_user
		self.filter_list_box = filter_list_box
		
		self.win = GtkWindow()
		self.win.set_title(_("Editing filter"))
		self.win.set_default_size(400,400)

		box = GtkVBox(spacing=5) ; box.set_border_width(5)
		self.win.add(box)
		box.show()

		# Disgusting over-use of GtkBoxes seems like a nice way
		# of constructing this ;-)
		box0 = GtkHBox(spacing=5) ; box0.set_border_width(5)
		box1 = GtkHBox(spacing=5) ; box1.set_border_width(5)
		box2 = GtkHBox(spacing=5) ; box2.set_border_width(5)
		box3 = GtkHBox(spacing=5) ; box3.set_border_width(5)
		box4 = GtkHBox()
		box4_1 = GtkVBox(spacing=5) ; box4_1.set_border_width(5)
		box4_2 = GtkVBox(spacing=5) ; box4_2.set_border_width(5)

		# Box0: filter name
		label = GtkLabel(_("Filter name: "))
		box0.pack_start(label, expand=FALSE)
		label.show()

		self.name_in = GtkEntry()
		box0.pack_start(self.name_in)
		self.name_in.show()

		box.pack_start(box0, expand=FALSE)
		box0.show()
	
		# Box1: if [text-in] <suggested headers>
		# eg:   if "subject"
		label = GtkLabel(_("If header"))
		box1.pack_start(label, expand=FALSE)
		label.show()
	
		menu = GtkMenu()
		menu.show()
		for x in range(0, len(f_headers)):
			menuitem = GtkMenuItem(f_headers[x])
			menuitem.set_data("0", x)
			menuitem.connect("activate", self.ch_header)
			menu.append(menuitem)
			menuitem.show()
		menuitem = GtkMenuItem("Custom:")
		menuitem.set_data("0", len(f_headers))
		menuitem.connect("activate", self.ch_header)
		menu.append(menuitem)
		menuitem.show()
			
		self.headmenu = GtkOptionMenu()
		self.headmenu.set_menu(menu)
		box1.pack_start(self.headmenu, expand=FALSE)
		self.headmenu.show()
		
		self.header_in = GtkEntry()
		box1.pack_start(self.header_in)
		self.header_in.show()
	
		box.pack_start(box1, expand=FALSE)
		box1.show()
	
		# Box2: <condition> <textin>
		# eg:   Equals "Here to torment you"
		menu = GtkMenu()
		menu.show()
		for x in range(0, len(f_conditions)):
			menuitem = GtkMenuItem(f_conditions[x])
			menuitem.set_data("0", x)
			menuitem.connect("activate", self.ch_condition)
			menu.append(menuitem)
			menuitem.show()
		self.cond_menu = GtkOptionMenu()
		self.cond_menu.set_menu(menu)
		box2.pack_start(self.cond_menu, expand=FALSE)
		self.cond_menu.show()
	
		self.carg_in = GtkEntry()
		box2.pack_end(self.carg_in)
		self.carg_in.show()
	
		box.pack_start(box2, expand=FALSE)
		box2.show()

		# Box3: then <action> <argument>
		# eg:   then "move to" deleted
		label = GtkLabel(_("then"))
		box3.pack_start(label, expand=FALSE)
		label.show()

		menu = GtkMenu()
		menu.show()
		for x in range(0, len(f_actions)):
			menuitem = GtkMenuItem(f_actions[x])
			menuitem.set_data("0", x)
			menuitem.connect("activate", self.ch_action)
			menu.append(menuitem)
			menuitem.show()
		self.act_menu = GtkOptionMenu()
		self.act_menu.set_menu(menu)
		box3.pack_start(self.act_menu, expand=FALSE)
		self.act_menu.show()

		self.aarg_in = GtkEntry()
		box3.pack_end(self.aarg_in)
		self.aarg_in.show()

		box.pack_start(box3, expand=FALSE)
		box3.show()

		# Box4_1: bunch of buttons and folder selector
		# for move_to rule
		self.folder_list = ptk.folder_tree.folder_tree()
		self.folder_list.update(self.parent_user.contents, update_all=1)
		self.folder_list.connect_button_press_event(self.sel_folder)
		swin = GtkScrolledWindow()
		swin.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
		swin.show()
		swin.add(self.folder_list)
		box4_1.pack_start(swin)#, expand=FALSE)
		swin.show()
		self.folder_list_win = swin

		def _save_filter(_button, self=self):
			self.f_name = self.name_in.get_text()
			if self.f_head < len(f_headers):
				# predefined header
				head_to_match = f_headers[self.f_head]
			else:
				# custom one
				head_to_match = self.header_in.get_text()
			# (if action is move then f_aarg is already a folder UID)
			if self.f_actn != ACTION_MOVE:
				self.f_aarg = self.aarg_in.get_text()
			self.f_carg = self.carg_in.get_text()
			
			self.filter = (self.f_name, head_to_match, self.f_cond, 
				                self.f_carg, self.f_actn, self.f_aarg)
			self.filter_list_box.filters[self.filter_index] = self.filter
			self.filter_list_box.update_filter_list()

		def _save_filter_close(_button, self=self, _save_filter=_save_filter):
			_save_filter(_button)
			# remove this window from the list
			self.filter_list_box.edit_windows.remove(self)
			self.win.destroy()
	
		# buttons horizontal in yet another box...
		button_box = GtkHBox(spacing=5)
		box.pack_end(button_box, expand=FALSE)
		button_box.show()

		button = ptk.misc_widgets.CancelButton()
		button.connect("clicked", self.win.destroy)
		button_box.pack_end(button, expand=FALSE)
		button.show()

		button = ptk.misc_widgets.ApplyButton()
		button.connect("clicked", _save_filter)
		button_box.pack_end(button, expand=FALSE)
		button.show()
		self.apply_button = button

		button = ptk.misc_widgets.OkButton()
		button.connect("clicked", _save_filter_close)
		button_box.pack_end(button, expand=FALSE)
		button.show()
		self.ok_button = button

		box.pack_start(box4)
		box4.show()
		box4.pack_start(box4_1, expand=FALSE)
		box4_1.show()

		# Load filter into widgets
		self.f_name = self.filter[0]
		if self.filter[1] in f_headers:
			self.f_head = f_headers.index(self.filter[1])
		else:
			# Custom...
			self.f_head = len(f_headers)
		self.f_cond = self.filter[2]
		self.f_carg = self.filter[3]
		self.f_actn = self.filter[4]
		self.f_aarg = self.filter[5]
		# Stuff the Gtk thingies with this junk
		self.name_in.set_text(self.f_name)
		self.headmenu.set_history(self.f_head)
		self.header_in.set_text(self.filter[1])
		self.carg_in.set_text(self.f_carg)
		self.act_menu.set_history(self.f_actn)
		self.cond_menu.set_history(self.f_cond)
		
		self.update_state()

		self.win.show()

class filters_editlist(GtkVBox):
	"""
	A GtkBox full of shit for editing a mailbox.filters
	"""
	def update_filter_list(self):
		self.filter_list.freeze()
		self.filter_list.clear()
		for x in self.filters:
			self.filter_list.append( [ x[0] ] )
		self.filter_list.thaw()

	def move_filter(self, old_pos, new_pos):
		# Correct filter index of edit windows
		for win in self.edit_windows:
			if win.filter_index == old_pos:
				win.filter_index = new_pos
			else:
				if win.filter_index >= new_pos:
					win.filter_index = win.filter_index + 1
				elif win.filter_index > old_pos and win.filter_index <= new_pos:
					win.filter_index = win.filter_index - 1
		
		filter = self.filters[old_pos]
		del self.filters[old_pos]
		self.filters.insert(new_pos, filter)
		self.update_filter_list()

	def __init__(self, mailbox, parent_user):
		GtkVBox.__init__(self, spacing=5)

		self.set_border_width(5)
		self.parent_user = parent_user

		import copy
		self.filters = copy.deepcopy(mailbox.filters)
		self.cur_edit = None	# currently editing which filter?

		# Windows open editing filters
		self.edit_windows = []

		# Last item selected by the user
		self.user_selected = []
		
		# Widgets to keep disabled until a filter is selected
		self.disabled_until_selection = []

		# buttons horizontal in yet another box...
		button_box = GtkHBox(spacing=5)
		self.pack_end(button_box, expand=FALSE)
		button_box.show()
		
		def new_filter(button, self=self):
			self.filters.append(("New filter","to",CONDITION_EQUAL,"",ACTION_MOVE,"deleted"))
			self.update_filter_list()
			self.edit_windows.append(filter_editbox(self.parent_user, self, len(self.filters)-1))
		
		button = GtkButton(" "+_("New")+" ")
		button.connect("clicked", new_filter)
		button_box.pack_end(button, expand=FALSE)
		button.show()

		def copy_filter(button, self=self):
			x = self.filter_list.selection[0]
			self.filters.append(self.filters[x])
			self.update_filter_list()
		
		button = GtkButton(" "+_("Copy")+" ")
		button.connect("clicked", copy_filter)
		button_box.pack_end(button, expand=FALSE)
		# disabled until you click on an item
		button.set_sensitive(FALSE)
		button.show()
		self.disabled_until_selection.append(button)

		def edit_filter(button, self=self):
			i = self.filter_list.selection[0]
			self.edit_windows.append(filter_editbox(self.parent_user, self, i))

		button = GtkButton(" "+_("Edit")+" ")
		button.connect("clicked", edit_filter)
		button_box.pack_end(button, expand=FALSE)
		# disabled until you click on an item
		button.set_sensitive(FALSE)
		button.show()
		self.disabled_until_selection.append(button)
			
		def delete_filter(button, self=self):
			index = self.filter_list.selection[0]
			# delete edit windows that are open and editing this filter
			for win in self.edit_windows:
				if win.filter_index == index:
					win.win.destroy()
					del win
				elif win.filter_index > index:
					win.filter_index = win.filter_index - 1
			del self.filters[index]
			self.update_filter_list()
			
		button = GtkButton(" "+_("Delete")+" ")
		button.connect("clicked", delete_filter)
		button_box.pack_end(button, expand=FALSE)
		# disabled until you click on an item
		button.set_sensitive(FALSE)
		button.show()
		self.disabled_until_selection.append(button)

		def _clicked_filter(_button, event, self=self):
			# What did we select?
			selected = self.filter_list.get_selection_info(event.x, event.y)
			if selected:
				# Enable some buttons
				for widget in self.disabled_until_selection:
					widget.set_sensitive(TRUE)
			else:
				for widget in self.disabled_until_selection:
					widget.set_sensitive(FALSE)
				return
			index = selected[0]
			self.user_selected = [index]
			# Open edit window on double-click
			if event.type == GDK._2BUTTON_PRESS:
				# Open filter edit box
				self.edit_windows.append(filter_editbox(self.parent_user, self, index))

		def _dnd_drag_data_get(w, context, selection_data, info, time, self=self):
			#
			selected = str(self.user_selected[0])
			selection_data.set(selection_data.target, 8, selected)
		
		def _dnd_drag_motion(w, context, x, y, time, self=self):
			# get target (pyne_filter)
			content_type = str(context.targets[0])
			y = y-20 # see mainwin for similar hack...
			selected = self.filter_list.get_selection_info(x,y)
			if selected:
				row = selected[0]
				self.filter_list.select_row(row, 0)

		def _dnd_drag_data_received(w, context, x, y, data, info, time, self=self):
			# get target
			content_type = str(context.targets[0])
			old_pos = int(data.data)
			if not self.filter_list.selection:
				return
			new_pos = self.filter_list.selection[0]
			# Move filter from old_pos to new_pos
			self.move_filter(old_pos, new_pos)

		targets = [ ("pyne_filter", 0, -1) ]
		# Box4_2: List of rules
		self.filter_list = GtkCList(1, [ _("Filters (highest evaluated first)") ])
		self.filter_list.connect("button_press_event", _clicked_filter)
		self.filter_list.connect("drag_data_received", _dnd_drag_data_received)
		self.filter_list.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_MOVE)
		self.filter_list.connect("drag_motion", _dnd_drag_motion)
		self.filter_list.connect("drag_data_get", _dnd_drag_data_get)
		self.filter_list.drag_source_set(GDK.BUTTON1_MASK|GDK.BUTTON3_MASK, targets, GDK.ACTION_MOVE)
		swin = GtkScrolledWindow()
		swin.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
		swin.show()
		swin.add(self.filter_list)
		self.filter_list.show()
		self.pack_start(swin)
		swin.show()

		# prepare for input
		self.update_filter_list()

